import type { HttpHandler, HttpRequestInit } from './client';
import { http } from './client';
type HttpMethod = 'GET' | 'POST' | 'PATCH' | 'PUT' | 'DELETE';
type HttpArgs = Record<string, string | number | boolean>;
type HttpDefaultClient = ReturnType<typeof http.default>;
type InferHttpHandlerConfig<T> = T extends HttpHandler<_Blob, infer C> ? C : never;
export interface RestOptions<THandler extends HttpHandler<Promise<object>, object>> {
    baseUrl: string;
    http: THandler;
    parseArgs: (args: Record<string, string | number | boolean>) => string;
    substituteRootParams: (path: string, args: Record<string, string | number | boolean>) => string;
}
export type RestSchema<T> = ((res: object) => T) | T;
export interface RestMemberBase<N extends string, M extends HttpMethod> {
    name: N;
    method: M;
}
export interface RestMethodMemberDef<TArgs = Record<string, string | number | boolean>, TRes = unknown, TConfig = Record<string, unknown>> {
    path: string;
    args?: TArgs;
    schema?: RestSchema<TRes>;
    config?: TConfig;
}
type DefaultHttpResult = any;
type _RestMember<TArgs = Record<string, string | number | boolean>, TRes = DefaultHttpResult> = RestMethodMemberDef<TArgs, TRes> & RestMemberBase<string, HttpMethod>;
export type RestMethodMemberCreator<TConfig, TMethod extends HttpMethod> = <TName extends string, TArgs = HttpArgs, TRes = object>(name: TName, opts?: Partial<RestMethodMemberDef<TArgs, TRes, TConfig>>) => RestMethodMemberDef<TArgs, TRes, TConfig> & {
    name: TName;
    method: TMethod;
};
export interface RestCreateFunctions<TConfig> {
    member<TName extends string, TMethod extends HttpMethod, TArgs = HttpArgs, TRes = object>(name: TName, method: TMethod, opts?: RestMethodMemberDef<TArgs, TRes, TConfig>): RestMethodMemberDef<TArgs, TRes, TConfig> & {
        name: TName;
        method: TMethod;
    };
    readonly get: RestMethodMemberCreator<TConfig, 'GET'>;
    readonly post: RestMethodMemberCreator<TConfig, 'POST'>;
    readonly patch: RestMethodMemberCreator<TConfig, 'PATCH'>;
    readonly put: RestMethodMemberCreator<TConfig, 'PUT'>;
    readonly del: RestMethodMemberCreator<TConfig, 'DELETE'>;
    readonly namespace: <TName extends string, TRest extends RestMember[]>(name: TName, createChildren: (fn: RestCreateFunctions<TConfig>) => TRest) => {
        name: TName;
        children: TRest;
    };
}
type InferRestRes<T> = T extends RestSchema<infer R> ? R : never;
interface _RestNamespace {
    name: string;
    children: RestMember[];
}
type RestMember = _RestMember | _RestNamespace;
export type RestMethodFn<TConfig, TArgs extends HttpArgs, TRes = unknown> = (args?: TArgs | null, config?: HttpRequestInit<TConfig>) => TRes;
type MapMember<M extends RestMember, TConfig> = M extends _RestNamespace ? {
    [K in M['name']]: InferRest<M['children'], TConfig>;
} : M extends _RestMember ? {
    [K in M['name']]: RestMethodFn<TConfig, NonNullable<M['args']>, Promise<InferRestRes<NonNullable<M['schema']>>>>;
} : never;
type MapMembers<T extends readonly RestMember[], TConfig> = {
    [K in keyof T]: T[K] extends RestMember ? MapMember<T[K], TConfig> : never;
}[number];
type MergeUnion<U> = (U extends unknown ? (k: U) => void : never) extends (k: infer I) => void ? {
    [K in keyof I]: I[K];
} : never;
export type InferRest<T extends readonly RestMember[], TConfig> = MergeUnion<MapMembers<T, TConfig>>;
export declare function rest<THandler extends HttpHandler<Promise<object>, object> = HttpDefaultClient>(restOptions?: Partial<RestOptions<THandler>>): <T extends readonly RestMember[]>(path: string, define: (fn: RestCreateFunctions<InferHttpHandlerConfig<THandler>>) => T) => InferRest<T, InferHttpHandlerConfig<THandler>>;
export {};
